﻿using System;
using KeePass.Plugins;
using KeePassLib.Keys;
using KeePassLib.Security;
using System.Diagnostics;
using WinKee.Forms;
using System.Windows.Forms;
using KeePass.App;

namespace WinKee
{
    public sealed class WinKeeExt : Plugin
    {
        private IPluginHost m_host = null;
        private WinKee m_provider = null;

        public override bool Initialize(IPluginHost host)
        {
            m_provider = new WinKee(host);
            m_host = host;
            m_host.KeyProviderPool.Add(m_provider);
            return true;
        }

        public override void Terminate()
        {
            m_provider.Clear();
            m_host.KeyProviderPool.Remove(m_provider);
        }

    }

    public sealed class WinKee : KeyProvider
    {

        public override bool DirectKey { get { return true; } }

        public override bool Exclusive { get { return true; } }

        public WinKee(IPluginHost host)
        {
            m_host = host;
            Dirty = false;
        }

        public override string Name
        {
            get { return "WinKee - locally encrypted access"; }
        }

        private IPluginHost m_host = null;
        private ProtectedBinary m_key = new ProtectedBinary();
        private bool m_firstRun = true;
        private bool m_AutoRun = false;

        public bool Dirty { get; private set; }

        public string DatabasePath { get; private set; }

        public override byte[] GetKey(KeyProviderQueryContext ctx)
        {
            if (m_firstRun &&
                this.Name == KeePass.Program.CommandLineArgs[AppDefs.CommandLineOptions.KeyFile])
            {
                m_firstRun = false;
                m_AutoRun = true;
                this.DatabasePath = KeePass.Program.CommandLineArgs.FileName;
            }
            else
            {
                m_AutoRun = false;
                this.DatabasePath = ctx.DatabasePath;
            }

            if (GetCredentials())
                return m_key.ReadData();
            else
                return null;
        }

        public void Clear()
        {
            //tb 06012012 m_key.Clear();
            m_key = new ProtectedBinary();
        }

        public bool GetCredentials()
        {
            WinKeeForm wkf = new WinKeeForm();

            string configBase = "WinKee.";
            string passwordField;
            string keypathField;
            string dbPath = this.DatabasePath.Replace(@"\", ".").Replace(":","");
            string strRef =
                Environment.MachineName + "." +
                Environment.UserDomainName + "." +
                Environment.UserName + "." +
                (dbPath != "" ? dbPath + "." : "")
                ;
            passwordField = configBase + strRef + "Password";
            keypathField = configBase + strRef + "KeyPath";

            if (m_host.CustomConfig.GetString(passwordField) != null)
            {
                wkf.EncryptedPassword =
                    m_host.CustomConfig.GetString(passwordField);
            }
            if (m_host.CustomConfig.GetString(keypathField) != null)
            {
                wkf.EncryptedKeyPath =
                    m_host.CustomConfig.GetString(keypathField);
            }

            wkf.AutoRun = this.m_AutoRun;

            DialogResult dr = wkf.ShowDialog();
            if (dr == DialogResult.Cancel) return false;
            else if (wkf.HasClosedWithExit)
            {
                Debug.Assert(dr == DialogResult.Abort);
                //tb OnFileExit(null, null);
                return false;
            }

            if (!Array.Equals(m_key.ReadData(), wkf.Key.ReadData()))
            {
                m_key = wkf.Key;
                m_host.CustomConfig.SetString(passwordField, wkf.EncryptedPassword);
                m_host.CustomConfig.SetString(keypathField, wkf.EncryptedKeyPath);
            }
            return true;
        }
    }
}